use aes;
use block_modes;
use block_modes::block_padding;
use block_modes::BlockMode;
use sha2;
use sha2::Digest;

type AesCbc = block_modes::Cbc<aes::Aes256, block_padding::Pkcs7>;

const SALT : &str = "LFsMH#kL!IfY:dcEz9F/dvj17nUN";

pub fn encrypt(plain_text : &str, password : &str, initial_value_byte_vector : &Vec<u8>) -> String
{
    let key_byte_vector : Vec<u8> = get_key_byte_vector(password);

    let cbc : block_modes::Cbc<aes::Aes256, block_padding::Pkcs7> = AesCbc::new_from_slices(&key_byte_vector, &initial_value_byte_vector).unwrap();

    let encrypted_byte_vector : Vec<u8> = cbc.encrypt_vec(plain_text.as_bytes());

    let encrypted_text : String =  base64::encode(encrypted_byte_vector);

    return encrypted_text;
}

pub fn decrypt(encrypted_text : &str, password : &str, initial_value_byte_vector : &Vec<u8>) -> String
{
    let key_byte_vector : Vec<u8> = get_key_byte_vector(password);

    let encrypted_byte_vector : Vec<u8> = base64::decode(encrypted_text).unwrap();

    let cbc : block_modes::Cbc<aes::Aes256, block_padding::Pkcs7> = AesCbc::new_from_slices(&key_byte_vector, initial_value_byte_vector).unwrap();

    let plain_byte_vector : Vec<u8> = cbc.decrypt_vec(&encrypted_byte_vector).unwrap();

    return String::from_utf8(plain_byte_vector).unwrap();
}

fn get_key_byte_vector(password : &str) -> Vec<u8>
{
    let password_salt : String = format!("{}::{}", password, SALT);

    let mut sha256 : sha2::Sha256 = sha2::Sha256::new();

    sha256.update(password_salt.as_bytes());

    return sha256.finalize().to_vec();
}

pub fn get_initial_value_byte_vector() -> Vec<u8>
{
    let mut initial_value_byte_vector : Vec<u8> = vec!
    [
        0, 0, 0, 0,
        0, 0, 0, 0,
        0, 0, 0, 0,
        0, 0, 0, 0
    ];

    getrandom::getrandom(&mut initial_value_byte_vector).unwrap();

    return initial_value_byte_vector;
}